import { ComponentFactoryResolver, Directive, ElementRef, EventEmitter, HostListener, Injector, Input, Output, ViewContainerRef } from '@angular/core';
import * as getCaretCoordinates from 'textarea-caret';
import 'rxjs/add/operator/takeUntil';
import { TextInputAutocompleteMenuComponent } from './text-input-autocomplete-menu.component';
var TextInputAutocompleteDirective = /** @class */ (function () {
    function TextInputAutocompleteDirective(componentFactoryResolver, viewContainerRef, injector, elm) {
        this.componentFactoryResolver = componentFactoryResolver;
        this.viewContainerRef = viewContainerRef;
        this.injector = injector;
        this.elm = elm;
        /**
         * The character that will trigger the menu to appear
         */
        this.triggerCharacter = '@';
        /**
         * The regular expression that will match the search text after the trigger character
         */
        this.searchRegexp = /^\w*$/;
        /**
         * The menu component to show with available options.
         * You can extend the built in `TextInputAutocompleteMenuComponent` component to use a custom template
         */
        this.menuComponent = TextInputAutocompleteMenuComponent;
        /**
         * Called when the options menu is shown
         */
        this.menuShown = new EventEmitter();
        /**
         * Called when the options menu is hidden
         */
        this.menuHidden = new EventEmitter();
        /**
         * Called when a choice is selected
         */
        this.choiceSelected = new EventEmitter();
        /**
         * A function that formats the selected choice once selected.
         */
        this.getChoiceLabel = function (choice) { return choice; };
    }
    TextInputAutocompleteDirective.prototype.onKeypress = function (e) {
        if (e.key === this.triggerCharacter) {
            this.showMenu();
        }
    };
    TextInputAutocompleteDirective.prototype.onChange = function (value) {
        var _this = this;
        if (this.menu) {
            if (value[this.menu.triggerCharacterPosition] !== this.triggerCharacter) {
                this.hideMenu();
            }
            else {
                var cursor = this.elm.nativeElement.selectionStart;
                if (cursor < this.menu.triggerCharacterPosition) {
                    this.hideMenu();
                }
                else {
                    var searchText = value.slice(this.menu.triggerCharacterPosition + 1, cursor);
                    if (!searchText.match(this.searchRegexp)) {
                        this.hideMenu();
                    }
                    else {
                        this.menu.component.instance.searchText = searchText;
                        this.menu.component.instance.choices = [];
                        this.menu.component.instance.choiceLoadError = undefined;
                        this.menu.component.instance.choiceLoading = true;
                        this.menu.component.changeDetectorRef.detectChanges();
                        Promise.resolve(this.findChoices(searchText))
                            .then(function (choices) {
                            if (_this.menu) {
                                _this.menu.component.instance.choices = choices;
                                _this.menu.component.instance.choiceLoading = false;
                                _this.menu.component.changeDetectorRef.detectChanges();
                            }
                        })
                            .catch(function (err) {
                            if (_this.menu) {
                                _this.menu.component.instance.choiceLoading = false;
                                _this.menu.component.instance.choiceLoadError = err;
                                _this.menu.component.changeDetectorRef.detectChanges();
                            }
                        });
                    }
                }
            }
        }
    };
    TextInputAutocompleteDirective.prototype.onBlur = function () {
        if (this.menu) {
            this.menu.lastCaretPosition = this.elm.nativeElement.selectionStart;
        }
    };
    TextInputAutocompleteDirective.prototype.showMenu = function () {
        var _this = this;
        if (!this.menu) {
            var menuFactory = this.componentFactoryResolver.resolveComponentFactory(this.menuComponent);
            this.menu = {
                component: this.viewContainerRef.createComponent(menuFactory, 0, this.injector),
                triggerCharacterPosition: this.elm.nativeElement.selectionStart
            };
            var lineHeight = +getComputedStyle(this.elm.nativeElement)
                .lineHeight.replace(/px$/, '');
            var _a = getCaretCoordinates(this.elm.nativeElement, this.elm.nativeElement.selectionStart), top_1 = _a.top, left = _a.left;
            this.menu.component.instance.position = {
                top: top_1 + lineHeight,
                left: left
            };
            this.menu.component.changeDetectorRef.detectChanges();
            this.menu.component.instance.closeMenu.subscribe(function (result) {
                _this.hideMenu();
            });
            this.menu.component.instance.selectChoice
                .takeUntil(this.menuHidden)
                .subscribe(function (choice) {
                var label = _this.getChoiceLabel(choice);
                var textarea = _this.elm.nativeElement;
                var value = textarea.value;
                var startIndex = _this.menu.triggerCharacterPosition;
                var start = value.slice(0, startIndex);
                var caretPosition = _this.menu.lastCaretPosition || textarea.selectionStart;
                var end = value.slice(caretPosition);
                textarea.value = start + label + end;
                // force ng model / form control to update
                textarea.dispatchEvent(new Event('input'));
                _this.hideMenu();
                var setCursorAt = (start + label).length;
                textarea.setSelectionRange(setCursorAt, setCursorAt);
                textarea.focus();
                _this.choiceSelected.emit({
                    choice: choice,
                    insertedAt: {
                        start: startIndex,
                        end: startIndex + label.length
                    }
                });
            });
            this.menuShown.emit();
        }
    };
    TextInputAutocompleteDirective.prototype.hideMenu = function () {
        if (this.menu) {
            this.menu.component.destroy();
            this.menuHidden.emit();
            this.menu = undefined;
        }
    };
    TextInputAutocompleteDirective.prototype.ngOnDestroy = function () {
        this.hideMenu();
    };
    TextInputAutocompleteDirective.decorators = [
        { type: Directive, args: [{
                    selector: 'textarea[mwlTextInputAutocomplete],input[type="text"][mwlTextInputAutocomplete]'
                },] },
    ];
    /** @nocollapse */
    TextInputAutocompleteDirective.ctorParameters = function () { return [
        { type: ComponentFactoryResolver, },
        { type: ViewContainerRef, },
        { type: Injector, },
        { type: ElementRef, },
    ]; };
    TextInputAutocompleteDirective.propDecorators = {
        'triggerCharacter': [{ type: Input },],
        'searchRegexp': [{ type: Input },],
        'menuComponent': [{ type: Input },],
        'menuShown': [{ type: Output },],
        'menuHidden': [{ type: Output },],
        'choiceSelected': [{ type: Output },],
        'findChoices': [{ type: Input },],
        'getChoiceLabel': [{ type: Input },],
        'onKeypress': [{ type: HostListener, args: ['keypress', ['$event'],] },],
        'onChange': [{ type: HostListener, args: ['input', ['$event.target.value'],] },],
        'onBlur': [{ type: HostListener, args: ['blur',] },],
    };
    return TextInputAutocompleteDirective;
}());
export { TextInputAutocompleteDirective };
//# sourceMappingURL=text-input-autocomplete.directive.js.map